// src/index.ts
import { createHighlighter } from "shiki";

// src/utils.ts
function debounce(func, {
  delay = 1500
} = {}) {
  let timer = void 0;
  return function(...args) {
    !timer && func.apply(this, args);
    timer = setTimeout(() => {
      timer = void 0;
    }, delay);
  };
}

// src/index.ts
function createPlainShiki(el, options) {
  const {
    lang,
    themes = {
      light: "min-light",
      dark: "min-dark"
    },
    selector = (theme) => `.${theme}`,
    watch = true,
    delay = 0,
    instance = createInstance()
  } = options;
  const isSupported = () => "CSS" in globalThis && "highlights" in CSS;
  const debouncedUpdate = debounce(update, { delay });
  const dispose = () => el.removeEventListener("input", debouncedUpdate);
  if (isSupported() && watch) {
    el.addEventListener("input", debouncedUpdate);
  }
  let getTokenLines;
  let stylesheet;
  Promise.resolve(instance).then((shiki) => {
    getTokenLines = (text) => shiki.codeToTokensWithThemes(text, {
      lang,
      themes
    });
    stylesheet = new CSSStyleSheet();
    document.adoptedStyleSheets.push(stylesheet);
    isSupported() && update();
  });
  const colorRanges = /* @__PURE__ */ new Map();
  function patch(loads) {
    const deleted = /* @__PURE__ */ new Set();
    for (const { token, range } of loads) {
      for (const theme in token.variants) {
        const { color } = token.variants[theme];
        const name = `shiki-${theme}-${color.slice(1).toLowerCase()}`;
        const isDefault = theme === "light";
        let highlight = CSS.highlights.get(name);
        if (!highlight) {
          CSS.highlights.set(name, highlight = new Highlight());
          highlight.priority = isDefault ? 0 : 1;
        }
        let ranges = colorRanges.get(name);
        if (!ranges) {
          const rule = `${isDefault ? ":root" : selector(theme)}::highlight(${name}) { color: ${color}; }`;
          stylesheet.insertRule(rule);
          colorRanges.set(name, ranges = []);
        }
        if (!deleted.has(name)) {
          deleted.add(name);
          for (const range2 of ranges) {
            highlight.delete(range2);
          }
          ranges.length = 0;
        }
        highlight.add(range);
        ranges.push(range);
      }
    }
  }
  function update() {
    const childNodes = el.childNodes;
    const textContent = [...childNodes].map((node) => node.textContent).join("");
    const tokenLines = getTokenLines(textContent);
    const loads = [];
    for (const tokens of tokenLines) {
      for (const token of tokens) {
        const [node, offset] = findNodeAndOffset(token.offset);
        const range = document.createRange();
        range.setStart(node, offset);
        range.setEnd(node, offset + token.content.length);
        loads.push({ token, range });
      }
    }
    patch(loads);
    function findNodeAndOffset(tokenOffset) {
      let offset = 0;
      for (const node of childNodes) {
        if (!node.textContent) {
          continue;
        }
        if (offset + node.textContent.length > tokenOffset) {
          return [node, tokenOffset - offset];
        } else {
          offset += node.textContent.length;
        }
      }
    }
  }
  return {
    get isSupported() {
      return isSupported();
    },
    dispose,
    update
  };
  async function createInstance() {
    const key = "__shiki_instance__";
    let instance2 = Reflect.get(globalThis, key);
    if (!instance2) {
      Reflect.set(
        globalThis,
        key,
        instance2 = await createHighlighter({
          langs: ["html", "css", "js", lang],
          themes: Object.values(themes)
        })
      );
    }
    return instance2;
  }
}
export {
  createPlainShiki
};
